#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <memory.h>
#include <string.h>

static void error_exit(const char* fmt, ...)
{
  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  exit(1);
}

#define MAX_N_WORDS 4096

// the binary image of the input SLD data
static union word {
  long l;
  float f;
} sld_words[MAX_N_WORDS];
static unsigned sld_n_words = 0;

//-----------------------------------------------------------------------------
// read a float in the SLD file and append it to the array sld_words.
// fp : input SLD file stream
// RETURN value : the float read from the file
static float read_float()
{
  float f;

  if(sld_n_words >= MAX_N_WORDS)
    error_exit("read_float : too many sld words.\n");

  if(scanf("%f", &f) != 1)
    error_exit("failed to read a float\n");

  return (sld_words[sld_n_words++].f = f);
}

//-----------------------------------------------------------------------------
// read an integer in the SLD file and append it to the array sld_words.
// RETURN value : the integer read from the file
static int read_int()
{
  long l;
  if(sld_n_words >= MAX_N_WORDS)
    error_exit("read_int : too many sld words.\n");

  if(scanf("%ld", &l) != 1)
    error_exit("failed to read an int\n");
  
  return (sld_words[sld_n_words++].l = l);
}

//-----------------------------------------------------------------------------
// read a 3D float vector and append it to the array sld_words.
static void read_vec3()
{
  read_float();
  read_float();
  read_float();
}

//-----------------------------------------------------------------------------
// read the scene environments
static void read_sld_env()
{
  // screen pos
  read_vec3();
  // screen rotation
  read_float();  read_float(); 
  // n_lights : Actually, it should be an int value !
  read_float();
  // light rotation
  read_float();  read_float();
  // beam  
  read_float();
}

//-----------------------------------------------------------------------------
// read all the objects
static void read_objects()
{

  while (read_int() != -1) {  // texture : -1 -> end
    // form
    read_int();
    // refltype
    read_int();
    // isrot_p
    int is_rot = read_int();
    // abc
    read_vec3();
    // xyz
    read_vec3();
    // is_invert
    read_float();
    // refl_param
    read_float(); read_float();
    // color
    read_vec3();
    // rot
    if(is_rot){
      read_vec3();
    }
  }
}

//-----------------------------------------------------------------------------
// read the AND-network
static void read_and_network()
{
  while(read_int() != -1){
    while(read_int() != -1);
  }
}

//-----------------------------------------------------------------------------
// read the OR-network
static void read_or_network()
{
  while(read_int() != -1){
    while(read_int() != -1);
  }
}

//-----------------------------------------------------------------------------
// read the SLD file
static void read_sld()
{
  read_sld_env();
  read_objects();
  read_and_network();
  read_or_network();
}

//-----------------------------------------------------------------------------
// main
int main(int argc, char* argv[])
{  

  read_sld();
  fwrite(sld_words, sizeof(union word), sld_n_words, stdout);
  return 0;
}

